from typing import Any
from typing_extensions import override
from uuid import uuid4, UUID

from eth_account import Account
from eth_account.datastructures import SignedMessage
from eth_account.messages import SignableMessage
from eth_account.signers.local import LocalAccount
from eth_typing import Hash32

from eth_hub.signatureinfo import SignatureInfo
from eth_hub.base_key_storage import BaseKeyStore
from eth_hub.local.key import LocalKey


class LocalKeyStorage(BaseKeyStore):
    def __init__(self) -> None:
        self._accounts: dict[UUID, LocalAccount] = {}

    @override
    def import_key(self, private_key: bytes) -> LocalKey:
        account: LocalAccount = Account.from_key(private_key)
        return self._add_account(account)

    @override
    def create_key(self) -> LocalKey:
        account: LocalAccount = Account.create()
        return self._add_account(account)

    @override
    def get_key(self, key_id: UUID) -> LocalKey:
        account: LocalAccount = self._accounts[key_id]
        return LocalKey(id=key_id, address=account.address)

    @override
    def list_keys(self) -> list[LocalKey]:
        return [
            LocalKey(id=key_id, address=account.address)
            for key_id, account in self._accounts.items()
        ]

    @override
    def remove_key(self, uuid: UUID) -> None:
        self._accounts.pop(uuid)

    @override
    def sign_hash(self, key_id: UUID, hash_: bytes) -> SignatureInfo:
        account: LocalAccount = self._accounts[key_id]
        hash32: Hash32 = Hash32(hash_)
        signed_message = account.unsafe_sign_hash(hash32)
        return self._get_signature_info(key_id, signed_message)

    @override
    def sign_message(
        self, key_id: UUID, signable_message: SignableMessage
    ) -> SignatureInfo:
        account: LocalAccount = self._accounts[key_id]
        signed_message = account.sign_message(signable_message)
        return self._get_signature_info(key_id, signed_message)

    @override
    def sign_transaction(
        self, key_id: UUID, transaction_data: dict[str, Any]
    ) -> SignatureInfo:
        account: LocalAccount = self._accounts[key_id]
        signature = account.sign_transaction(transaction_data)
        return SignatureInfo(
            key_id=key_id,
            hash=signature.hash,
            v=signature.v,
            r=f"0x{signature.r:064x}",
            s=f"0x{signature.s:064x}",
        )

    def _add_account(self, account: LocalAccount) -> LocalKey:
        key_id: UUID = uuid4()
        self._accounts[key_id] = account

        return LocalKey(id=key_id, address=account.address)

    def _get_signature_info(
        self, key_id: UUID, signed_message: SignedMessage
    ) -> SignatureInfo:
        return SignatureInfo(
            key_id=key_id,
            hash=signed_message.message_hash,
            v=signed_message.v,
            r=f"0x{signed_message.r:064x}",
            s=f"0x{signed_message.s:064x}",
        )



import uuid
from unittest.mock import Mock


from eth_hub.signatureinfo import SignatureInfo
from eth_hub.local.key_storage import LocalKeyStorage


def test_create_key(account_mock: Mock, signed_tx_mock: Mock) -> None:
    # given
    key_id = uuid.uuid4()
    local_signer = LocalKeyStorage()
    local_signer._accounts[key_id] = account_mock

    # when
    transaction_data = {
        "nonce": 123,
        "gas_price": 123,
        "gas_limit": 123,
        "to": "0x09616C3d61b3331fc4109a9E41a8BDB7d9776609",
        "chain_id": 1,
        "value": 123
    }
    signature = local_signer.sign_transaction(
        key_id=key_id,
        transaction_data=transaction_data
    )

    # then
    assert isinstance(signature, SignatureInfo)
    assert signature.key_id == key_id
    assert signature.hash == signed_tx_mock.hash.encode()
    assert signature.v == signed_tx_mock.v
    assert signature.r == f"0x{signed_tx_mock.r:064x}"
    assert signature.s == f"0x{signed_tx_mock.s:064x}"
    account_mock.sign_transaction.assert_called_once()



import datetime
from enum import StrEnum
from typing import Optional
from uuid import UUID

from pydantic import BaseModel, Field


class ResponseMetadata(BaseModel):
    http_status_code: int = Field(..., alias="HTTPStatusCode")


class Error(BaseModel):
    code: str = Field(..., alias="Code")
    message: str = Field(..., alias="Message")


class Response(BaseModel):
    response_metadata: ResponseMetadata = Field(..., alias="ResponseMetadata")
    error: Optional[Error] = Field(None, alias="Error")


class KeyState(StrEnum):
    CREATING = "Creating"
    ENABLED = "Enabled"
    DISABLED = "Disabled"
    PENDING_DELETION = "PendingDeletion"
    PENDING_IMPORT = "PendingImport"
    PENDING_REPLICA_DELETION = "PendingReplicaDeletion"
    UNAVAILABLE = "Unavailable"
    UPDATING = "Updating"


class KeySpec(StrEnum):
    RSA_2048 = "RSA_2048"
    RSA_3072 = "RSA_3072"
    RSA_4096 = "RSA_4096"
    ECC_NIST_P256 = "ECC_NIST_P256"
    ECC_NIST_P384 = "ECC_NIST_P384"
    ECC_NIST_P521 = "ECC_NIST_P521"
    ECC_SECG_P256K1 = "ECC_SECG_P256K1"
    SYMMETRIC_DEFAULT = "SYMMETRIC_DEFAULT"
    HMAC_224 = "HMAC_224"
    HMAC_256 = "HMAC_256"
    HMAC_384 = "HMAC_384"
    HMAC_512 = "HMAC_512"
    SM2 = "SM2"


class KeyMetadata(BaseModel):
    key_id: UUID = Field(..., alias="KeyId")
    enabled: bool = Field(..., alias="Enabled")
    key_state: KeyState = Field(..., alias="KeyState")
    description: str = Field(..., alias="Description")
    key_spec: KeySpec = Field(..., alias="CustomerMasterKeySpec")


class AliasItem(BaseModel):
    name: str = Field(..., alias="AliasName")
    target_key_id: UUID = Field(..., alias="TargetKeyId")


class KeyItem(BaseModel):
    key_id: UUID = Field(..., alias="KeyId")


class ListAliasesPage(Response):
    aliases: list[AliasItem] = Field(..., alias="Aliases")


class ListKeysPage(Response):
    keys: list[KeyItem] = Field(..., alias="Keys")


class CreateKeyItemResponse(Response):
    key_metadata: KeyMetadata = Field(..., alias="KeyMetadata")


class GetPublicKeyResponse(Response):
    public_key: bytes = Field(..., alias="PublicKey")


class DescribeKeyResponse(Response):
    key_metadata: KeyMetadata = Field(..., alias="KeyMetadata")


class ScheduleDeletionResponse(Response):
    key_arn: str = Field(..., alias="KeyId")
    deletion_datetime: datetime.datetime = Field(..., alias="DeletionDate")
    key_state: KeyState = Field(..., alias="KeyState")
    days_period: int = Field(..., alias="PendingWindowInDays")


class SignResponse(Response):
    key_arn: str = Field(..., alias="KeyId")
    signature: bytes = Field(..., alias="Signature")
    signing_algorithm: bytes = Field(..., alias="SigningAlgorithm")



from pydantic import BaseModel, UUID4, PositiveInt, Field


class SignatureInfo(BaseModel):
    key_id: UUID4
    hash: bytes
    v: PositiveInt = Field(..., ge=27, examples=[27])
    r: str = Field(
        ...,
        max_length=66,
        min_length=66,
        json_schema_extra={"format": "hexadecimal"},
        examples=[f"0x{0:0>64}"],
    )
    s: str = Field(
        ...,
        max_length=66,
        min_length=66,
        json_schema_extra={"format": "hexadecimal"},
        examples=[f"0x{0:0>64}"],
    )



from uuid import UUID

from botocore.exceptions import ClientError

from eth_hub.aws.boto3_wrappers.dto import (
    ListAliasesPage,
    DescribeKeyResponse,
    KeyMetadata,
)
from eth_hub.aws.boto3_wrappers.exceptions import (
    CantCreateKeyObjectAwsError,
    CantListAliasesAwsError,
    CantGetKeyInfoAwsError,
)
from mypy_boto3_kms import KMSClient


def set_alias(client: KMSClient, key_id: UUID, alias: str) -> None:
    try:
        client.create_alias(
            AliasName=f"alias/{alias}",
            TargetKeyId=str(key_id),
        )
    except ClientError as exception:
        raise CantCreateKeyObjectAwsError(exception)  # from exception


def get_aliases(client: KMSClient, key_id: UUID) -> list[str]:
    try:
        response = client.list_aliases(KeyId=str(key_id))
    except ClientError as error:
        raise CantListAliasesAwsError(error)

    page = ListAliasesPage.model_validate(response)
    return [alias.name.removeprefix("alias/") for alias in page.aliases]


def check_alias_already_taken(client: KMSClient, alias: str) -> bool:
    try:
        paginator = client.get_paginator("list_aliases")
        return any(
            alias_item.name == f"alias/{alias}"
            for page in paginator.paginate()
            for alias_item in ListAliasesPage.model_validate(page).aliases
        )
    except ClientError as error:
        raise CantListAliasesAwsError(error)


def get_key_metadata(client: KMSClient, key_id: UUID) -> KeyMetadata:
    try:
        response_raw = client.describe_key(KeyId=str(key_id))
    except ClientError as error:
        raise CantGetKeyInfoAwsError(error)

    response = DescribeKeyResponse.model_validate(response_raw)
    return response.key_metadata



from eth_hub.aws.boto3_wrappers.exceptions import BaseAwsError


class BaseError(Exception):
    pass


class AliasAlreadyTakenError(BaseError):
    pass


class KeyNotFound(BaseError):
    pass


class CantFindValidVError(BaseError):
    pass


class BaseFromAwsError(BaseError):
    message = "Base error"

    def __init__(self, aws_error: BaseAwsError) -> None:
        self.code = aws_error.code
        super().__init__(f"{self.message}: {self.code}")


class CantGetKeyInfoError(BaseFromAwsError):
    message = "Can't get key info"


class CantListKeysError(BaseFromAwsError):
    message = "Can't list keys"


class CantCreateKeyError(BaseFromAwsError):
    message = "Can't create key"


class CantSetAlias(BaseFromAwsError):
    message = "Can't set alias"


class CantSignHash(BaseFromAwsError):
    message = "Can't sign hash"


class CantRemoveKey(BaseFromAwsError):
    message = "Can't remove key"



import datetime
import uuid

from botocore.stub import Stubber
from mypy_boto3_kms import KMSClient

from eth_hub.aws.boto3_wrappers.dto import KeyState, KeySpec
from eth_hub.aws.boto3_wrappers.metadata import set_alias, check_alias_already_taken, \
    get_key_metadata, get_aliases


def test_set_alias(client: KMSClient, stubber: Stubber) -> None:
    # given
    key_id = uuid.uuid4()

    stubber.add_response(
        method="create_alias",
        service_response={
            "ResponseMetadata": {
                "HTTPStatusCode": 200
            }
        }
    )

    # when
    set_alias(client, key_id, "test")

    # then
    stubber.assert_no_pending_responses()


def test_get_aliases(client: KMSClient, stubber: Stubber) -> None:
    # given
    key_id = uuid.uuid4()
    alias1 = "test1"
    alias2 = "test2"

    stubber.add_response(
        method="list_aliases",
        service_response={
            "Aliases": [
                {
                    "AliasName": f"alias/{alias1}",
                    "TargetKeyId": str(key_id),
                }, {
                    "AliasName": f"alias/{alias2}",
                    "TargetKeyId": str(key_id),
                }
            ],
            "ResponseMetadata": {
                "HTTPStatusCode": 200
            }
        }
    )

    # when
    aliases = get_aliases(client, key_id)

    # then
    stubber.assert_no_pending_responses()
    assert aliases == [alias1, alias2]


def test_check_alias_already_taken(client: KMSClient, stubber: Stubber) -> None:
    # given
    key_id = uuid.uuid4()
    alias1 = "test1"
    alias2 = "test2"

    stubber.add_response(
        method="list_aliases",
        service_response={
            "Aliases": [
                {
                    "AliasName": f"alias/{alias1}",
                    "TargetKeyId": str(key_id),
                }, {
                    "AliasName": f"alias/{alias2}",
                    "TargetKeyId": str(key_id),
                }
            ],
            "ResponseMetadata": {
                "HTTPStatusCode": 200
            }
        }
    )

    # when
    alread_taken = check_alias_already_taken(client, alias1)

    # then
    stubber.assert_no_pending_responses()
    assert alread_taken is True


def test_get_key_metadata(client: KMSClient, stubber: Stubber) -> None:
    # given
    key_id = uuid.uuid4()
    enabled = True
    state = KeyState.ENABLED
    description = "Test Key"
    creation_date = datetime.datetime.now(datetime.UTC)

    stubber.add_response(
        method="describe_key",
        service_response={
            "KeyMetadata": {
                "KeyId": str(key_id),
                "Enabled": enabled,
                "KeyState": state.value,
                "Description": description,
                "CustomerMasterKeySpec": KeySpec.ECC_SECG_P256K1.value,
                "CreationDate": creation_date,
            },
            "ResponseMetadata": {
                "HTTPStatusCode": 200
            }
        }
    )

    # when
    key = get_key_metadata(client, key_id)

    # then
    stubber.assert_no_pending_responses()
    assert key_id == key.key_id
    assert key.enabled == enabled
    assert key.description == description



import uuid
import datetime

import pytest

from botocore.stub import Stubber
from mypy_boto3_kms import KMSClient
from pydantic import SecretBytes

from eth_hub.aws.boto3_wrappers.dto import KeyState, KeySpec
from eth_hub.aws.boto3_wrappers.key import (
    create_key_item,
    fulfil_private_key,
    get_key_ids,
    get_address,
    schedule_key_deletion
)
from eth_hub.aws.boto3_wrappers.exceptions import BaseAwsError


WRAPPING_KEY = bytes.fromhex("""
30820122300d06092a864886f70d01010105000382010f003082010a0282010100e4823c943c43a114fb77
b7776b62a8fbf80c51e4df2776c49e804f847b56d01ac688223af6d1d58172b4ddf6d24c0d6af06c4befe8
1439d3e3f3308d5bcfa5c7f9f080b53c3409a8056253b2dad7ccdee5ef7b6bf199f1e4b8df341e9155382f
52045236aee0225e153cd492ca99cda93905b9070a47e20d6680272e854b7b9911958f3b2dbe2495564a7b
9831071b2a60351103614fbaf21b51e09add4a3952a87c23188182837665bce1f54a6dc86c05116e249b71
60b03a48adc8335f64ee59d81f3b2f33b7f149b43d88f7409d572e03d9e717fce57795847077203e813cef
703edcc29d2af3d8d4f2a4dedc2bcc2d2d0cf9ad31d7542cf69dc7b6c279b90203010001
""")


def test_create_key_item(client: KMSClient, stubber: Stubber) -> None:
    # given
    key_id = uuid.uuid4()

    stubber.add_response(
        method="create_key",
        service_response={
            "KeyMetadata": {
                "KeyId": str(key_id),
                "Enabled": True,
                "KeyState": KeyState.ENABLED.value,
                "Description": "Test Key",
                "CustomerMasterKeySpec": KeySpec.ECC_SECG_P256K1.value,
                "CreationDate": datetime.datetime.now(datetime.UTC),
            },
            "ResponseMetadata": {
                "HTTPStatusCode": 200
            }
        }
    )

    # when
    retrieved_key_id = create_key_item(client, create_key_by_aws=True)

    # then
    assert retrieved_key_id == key_id
    stubber.assert_no_pending_responses()


def test_create_key_item_error(client: KMSClient, stubber: Stubber) -> None:
    # given
    stubber.add_client_error(
        method="create_key",
        service_error_code="AccessDeniedException",
        service_message="You do not have sufficient permissions to perform this action.",
        http_status_code=403
    )

    # when / then
    with pytest.raises(BaseAwsError):
        create_key_item(client, create_key_by_aws=True)

    stubber.assert_no_pending_responses()

def test_fulfil_private_key(client: KMSClient, stubber: Stubber) -> None:
    # given
    key_id = uuid.uuid4()

    stubber.add_response(
        method="get_parameters_for_import",
        service_response={
            "KeyId": f"arn:aws:kms:eu-west-2:222222222222:key/{key_id}",
            "ImportToken": b"test",
            "PublicKey": WRAPPING_KEY,
            "ParametersValidTo": "2025-02-04T14:30:00.000000",
            "ResponseMetadata": {
                "HTTPStatusCode": 200
            }
        }
    )
    stubber.add_response(
        method="import_key_material",
        service_response={
        "ResponseMetadata": {
                "HTTPStatusCode": 200
            }
        }
    )

    # when
    fulfil_private_key(client=client, key_id=key_id, private_key=SecretBytes(b"1"))

    # then
    stubber.assert_no_pending_responses()


def test_get_key_ids(client: KMSClient, stubber: Stubber) -> None:
    # given
    key_id_1 = uuid.uuid4()
    key_id_2 = uuid.uuid4()

    stubber.add_response(
        method="list_keys",
        service_response={
            "Keys": [
                {
                    "KeyId": str(key_id_1),
                    "KeyArn": f"arn:aws:kms:eu-west-2:111111111111:key/{key_id_1}"
                }
            ],
            "NextMarker": "next-marker-1",  # Correct pagination key for KMS
            "Truncated": True,
            "ResponseMetadata": {
                "HTTPStatusCode": 200
            }
        }
    )
    stubber.add_response(
        method="list_keys",
        service_response={
            "Keys": [
                {
                    "KeyId": str(key_id_2),
                    "KeyArn": f"arn:aws:kms:eu-west-2:111111111111:key/{key_id_2}"
                }
            ],
            "Truncated": False,
            "ResponseMetadata": {
                "HTTPStatusCode": 200
            }
        }
    )

    # when
    key_ids = get_key_ids(client)

    # then
    stubber.assert_no_pending_responses()
    assert key_ids == [key_id_1, key_id_2]


def test_get_address(client: KMSClient, stubber: Stubber) -> None:
    # given
    key_id = uuid.uuid4()
    public_key = bytes.fromhex(
        "3056301006072a8648ce3d020106052b8104000a034200042d3665dce3d34dfdbcc585ffdee7f7b245f01685"
        "d228a3d1c04ea80805282c54bf66263c4c0d2a8c0bdb0ba0f888d1f9b4a8b14325d8b39492b684afcb16e1e6"
    )
    address = "9c60ee98a12d30ca9eaccc9635a10864756a00d7"

    stubber.add_response(
        method="get_public_key",
        service_response={
            "PublicKey": public_key,
            "ResponseMetadata": {
                "HTTPStatusCode": 200
            }
        }
    )

    # when
    address_received = get_address(client, key_id=key_id)

    # then
    stubber.assert_no_pending_responses()
    assert address_received.hex() == address


def test_schedule_key_deletion(client: KMSClient, stubber: Stubber) -> None:
    # given
    key_id = uuid.uuid4()
    deletion_date = datetime.datetime.now(datetime.UTC)

    stubber.add_response(
        method="schedule_key_deletion",
        service_response={
            "DeletionDate": deletion_date,
            "KeyId": f"arn:aws:kms:eu-west-2:237149049208:key/{key_id}",
            "KeyState": "PendingDeletion",
            "PendingWindowInDays": 7,
            "ResponseMetadata": {
                "HTTPStatusCode": 200,
            }
        }
    )

    # when
    deletion_date_received = schedule_key_deletion(client, key_id=key_id)

    # then
    stubber.assert_no_pending_responses()
    assert deletion_date == deletion_date_received



import uuid
from unittest.mock import Mock

from eth_account.messages import encode_defunct

from eth_hub.signatureinfo import SignatureInfo
from eth_hub.local.key_storage import LocalKeyStorage


def test_create_key(account_mock: Mock, signed_message_mock: Mock) -> None:
    # given
    key_id = uuid.uuid4()
    local_signer = LocalKeyStorage()
    local_signer._accounts[key_id] = account_mock
    message = encode_defunct(text="test message")

    # when
    signature = local_signer.sign_message(key_id=key_id, signable_message=message)

    # then
    assert isinstance(signature, SignatureInfo)
    assert signature.key_id == key_id
    assert signature.hash == signed_message_mock.message_hash.encode()
    assert signature.v == signed_message_mock.v
    assert signature.r == f"0x{signed_message_mock.r:064x}"
    assert signature.s == f"0x{signed_message_mock.s:064x}"
    account_mock.sign_message.assert_called_once()



import pytest

from eth_hub.aws.boto3_wrappers.exceptions import CantCreateKeyObjectAwsError, CantImportKeyMaterialAwsError
from eth_hub.aws.exceptions import CantCreateKeyError
from eth_hub.aws.key_store import AwsKeyStore
from .aws_mock import AwsMock


def test_create_key(
        key_manager: AwsKeyStore,
        aws_mock: AwsMock,
) -> None:
    # when
    key_manager.create_key()

    # then
    aws_mock.create_key_item_mock.assert_called_once()
    aws_mock.fulfil_private_key_mock.assert_not_called()


def test_create_key_with_exception(
        key_manager: AwsKeyStore,
        aws_mock: AwsMock,
) -> None:
    # given
    aws_mock.create_key_item_mock.side_effect = CantCreateKeyObjectAwsError

    # when / then
    with pytest.raises(CantCreateKeyError):
        key_manager.create_key()


def test_import_key(
        key_manager: AwsKeyStore,
        aws_mock: AwsMock,
) -> None:
    # given
    pk = bytes.fromhex("b8c0e2a59eb900206b21ad19b632725b0e113fb46cef49ed3d418cba056dbd00")

    # when
    key_manager.import_key(pk)

    # then
    aws_mock.create_key_item_mock.assert_called_once()
    aws_mock.fulfil_private_key_mock.assert_called_once()


def test_import_key_with_exception_on_key_creation(
        key_manager: AwsKeyStore,
        aws_mock: AwsMock,
) -> None:
    # given
    pk = bytes.fromhex("b8c0e2a59eb900206b21ad19b632725b0e113fb46cef49ed3d418cba056dbd00")
    aws_mock.create_key_item_mock.side_effect = CantCreateKeyObjectAwsError

    # when / then
    with pytest.raises(CantCreateKeyError):
        key_manager.import_key(pk)


def test_import_key_with_exception_on_fulfilling_pk(
        key_manager: AwsKeyStore,
        aws_mock: AwsMock,
) -> None:
    # given
    pk = bytes.fromhex("b8c0e2a59eb900206b21ad19b632725b0e113fb46cef49ed3d418cba056dbd00")
    aws_mock.fulfil_private_key_mock.side_effect = CantImportKeyMaterialAwsError

    # when / then
    with pytest.raises(CantCreateKeyError):
        key_manager.import_key(pk)



from eth_account import Account

from eth_hub.local.key_storage import LocalKeyStorage


def test_create_key(private_key: str) -> None:
    # given
    local_signer = LocalKeyStorage()
    account = Account.create()

    # when
    key_info = local_signer.import_key(private_key=account._private_key)

    # then
    assert key_info.id in local_signer._accounts
    assert key_info.address == account.address



import datetime
from typing import Optional
from uuid import UUID, uuid4

from eth_account import Account
from eth_account.signers.local import LocalAccount
from mypy_boto3_kms.client import KMSClient
from pydantic import BaseModel, SecretBytes, ConfigDict
from pytest_mock import MockerFixture

from eth_hub.aws.boto3_wrappers.dto import KeyState, KeyMetadata, KeySpec
from eth_hub.aws.boto3_wrappers.exceptions import CantGetAddressAwsError, CantCreateKeyObjectAwsError


class Key(BaseModel):
    model_config = ConfigDict(arbitrary_types_allowed=True)

    id: UUID
    aliases: list[str] = []
    account: Optional[LocalAccount] = None
    key_state: KeyState = KeyState.ENABLED
    removal_scheduled: bool = False
    enabled: bool = True


class AwsMock:
    def __init__(self, mocker: MockerFixture) -> None:
        self.keys: dict[UUID, Key] = {}

        self.create_key_item_mock = mocker.patch(
            "eth_hub.aws.key_store.create_key_item",
            side_effect=self.create_key_item
        )

        self.fulfil_private_key_mock = mocker.patch(
            "eth_hub.aws.key_store.fulfil_private_key",
            side_effect=self.fulfil_private_key
        )

        self.get_key_ids_mock = mocker.patch(
            "eth_hub.aws.key_store.get_key_ids",
            side_effect=self.get_key_ids
        )

        self.get_address_mock = mocker.patch(
            "eth_hub.aws.key_store.get_address",
            side_effect=self.get_address
        )

        self.schedule_key_deletion_mock = mocker.patch(
            "eth_hub.aws.key_store.schedule_key_deletion",
            side_effect=self.schedule_key_deletion
        )

        self.set_alias_mock = mocker.patch(
            "eth_hub.aws.key_store.set_alias",
            side_effect=self.set_alias
        )

        self.get_aliases_mock = mocker.patch(
            "eth_hub.aws.key_store.get_aliases",
            side_effect=self.get_aliases
        )

        self.check_alias_already_taken_mock = mocker.patch(
            "eth_hub.aws.key_store.check_alias_already_taken",
            side_effect=self.check_alias_already_taken
        )

        self.get_key_metadata_mock = mocker.patch(
            "eth_hub.aws.key_store.get_key_metadata",
            side_effect=self.get_key_metadata
        )

    def create_key_item(self, client: KMSClient, create_key_by_aws: bool) -> UUID:
        key_id = uuid4()
        self.keys[key_id] = Key(id=key_id)
        if create_key_by_aws:
            self.keys[key_id].account = Account.create()

        return key_id

    def fulfil_private_key(self, client: KMSClient, key_id: UUID, private_key: SecretBytes) -> None:
        self.keys[key_id].account = Account.from_key(private_key.get_secret_value())

    def get_key_ids(self, client: KMSClient) -> list[UUID]:
        return list(self.keys.keys())

    def get_address(self, client: KMSClient, key_id: UUID) -> bytes:
        if (key := self.keys[key_id]) and key.account is not None:
            return bytes.fromhex(key.account.address.removeprefix("0x"))

        raise CantGetAddressAwsError()

    def schedule_key_deletion(self, client: KMSClient, key_id: UUID, days_period=7) -> datetime.datetime:
        self.keys[key_id].removal_scheduled = True
        return datetime.datetime.now(datetime.UTC) + datetime.timedelta(days=days_period)

    def set_alias(self, client: KMSClient, key_id: UUID, alias: str) -> None:
        if self.check_alias_already_taken(client, alias):
            raise CantCreateKeyObjectAwsError()

        self.keys[key_id].aliases.append(alias)

    def get_aliases(self, client: KMSClient, key_id: UUID) -> list[str]:
        return self.keys[key_id].aliases

    def check_alias_already_taken(self, client: KMSClient, alias: str) -> bool:
        return any(alias in key.aliases for key in list(self.keys.values()))

    def get_key_metadata(self, client: KMSClient, key_id: UUID) -> KeyMetadata:
        return KeyMetadata(
            KeyId=key_id,
            Enabled=self.keys[key_id].enabled,
            KeyState=self.keys[key_id].key_state,
            Description="",
            CustomerMasterKeySpec=KeySpec.ECC_SECG_P256K1
        )



import pytest

from eth_hub.aws.boto3_wrappers.exceptions import CantCreateKeyObjectAwsError
from eth_hub.aws.exceptions import CantSetAlias, AliasAlreadyTakenError
from eth_hub.aws.key_store import AwsKeyStore
from .aws_mock import AwsMock, Key


def test_set_alias(
        mocked_key: Key,
        key_manager: AwsKeyStore,
        aws_mock_with_key: AwsMock,
) -> None:
    # given
    alias = "test"

    # when
    key_manager.set_alias(mocked_key.id, alias)

    # then
    aws_mock_with_key.set_alias_mock.assert_called_once()
    assert alias in aws_mock_with_key.keys[mocked_key.id].aliases


def test_set_already_taken_alias(
        mocked_key: Key,
        key_manager: AwsKeyStore,
        aws_mock_with_key: AwsMock,
) -> None:
    # given
    alias = "test"
    mocked_key.aliases.append(alias)

    # when
    with pytest.raises(AliasAlreadyTakenError):
        key_manager.set_alias(mocked_key.id, alias)


def test_set_alias_with_exception(
        mocked_key: Key,
        key_manager: AwsKeyStore,
        aws_mock_with_key: AwsMock,
) -> None:
    # given
    alias = "test"
    aws_mock_with_key.set_alias_mock.side_effect = CantCreateKeyObjectAwsError

    # when
    with pytest.raises(CantSetAlias):
        key_manager.set_alias(mocked_key.id, alias)
